package org.honours.gui;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;

import org.expeditee.actions.Misc;
import org.expeditee.gui.DisplayIO;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameIO;
import org.expeditee.gui.FrameKeyboardActions;
import org.expeditee.gui.MessageBay;
import org.expeditee.gui.UserSettings;
import org.expeditee.items.Item;
import org.expeditee.items.Text;
import org.expeditee.items.widgets.WidgetCorner;
import org.expeditee.stats.Formatter;
import org.honours.actions.MiscActions;
import org.honours.collection.CollectionItem;
import org.honours.collection.NoteLayer;

public class HonoursFrameIO {

	private static Frame _lastCopiedFrame = null;	
	
	public static boolean DISPLAY_TOOLBAR = true;
	public static boolean DISPLAY_LAYERS = true;
	public static boolean DISPLAY_NOTES = false;
	
	public static boolean DISPLAY_TOGGLER = true;
	
	public static String COLLECTIONS_PATH = FrameIO.PARENT_FOLDER +
			"collections" + File.separator;
	
	/**
	 * Generate a new frame for a given (existing) 
	 * frameset.
	 * @param frameNumber - frame number for this next item.
	 * @param frameset - frameset to add frame to.
	 */
	public static void generateFrame(int frameNumber, String frameset){
		String frameFileName = FrameIO.FRAME_PATH + frameset + File.separator +
				frameNumber + ".exp";
		Frame checkExistingFrame = FrameIO.LoadFrame(frameset + frameNumber);
		
		if(checkExistingFrame == null){
			FileWriter fw;
			BufferedWriter bw;
			
			try{
				fw = new FileWriter(frameFileName);
				bw = new BufferedWriter(fw);
				writeOutFrameHeader(bw);
				FrameIO.WriteINF(FrameIO.FRAME_PATH, frameset, frameset+frameNumber);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Writes out a frame header for a new frame.
	 * @param bw
	 */
	public static void writeOutFrameHeader(BufferedWriter bw){
		try {
			StringBuffer header = new StringBuffer("");
			
			header.append("V 1\n");
			header.append("p 44");
			header.append("U " + UserSettings.UserName + "\n");
			header.append("D " + Formatter.getDateTime() + "\n");	//
			header.append("M " + UserSettings.UserName + "\n");		//last modified by
			header.append("d " + Formatter.getDateTime() + "\n");
			header.append("Z\n\n");
			bw.write(header.toString());
			
		}catch(Exception e) {
			e.printStackTrace();
		}
	}

	public static Frame CopyFrame() {
		_lastCopiedFrame = DisplayIO.getCurrentFrame();
		MessageBay.displayMessage("Item successfully copied.");
		
		return _lastCopiedFrame;
	}

	public static Frame PasteFrame() {
		
		Frame current = DisplayIO.getCurrentFrame();
		if(_lastCopiedFrame != null){
			
			List<Item> list = _lastCopiedFrame.getVisibleItems();
			List<Item> copy = new ArrayList<Item>();
			
			for(Item item : list){
				if(!(item instanceof WidgetCorner))
					copy.add(item);
			}
			
			current.addAllItems(copy);
			
		}else{
			MessageBay.errorMessage("ERROR: NO FRAME/ITEM TO COPY");
			return null;
		}
		
		return _lastCopiedFrame;

	}
	
	/**
	 * Toggles on and off the display of the "Toggle 
	 * Toolbar" overlay.
	 */
	public static void toggleToggler(){
		if(DISPLAY_TOGGLER){
			swapLinks(Interfaces.TOGGLE_TOOLS_OVERLAY,Interfaces.BLANK_TOGGLER_OVERLAY);
			DISPLAY_TOGGLER = false;
		}else{
			swapLinks(Interfaces.BLANK_TOGGLER_OVERLAY,Interfaces.TOGGLE_TOOLS_OVERLAY);
			DISPLAY_TOGGLER = true;
		}
		FrameKeyboardActions.Refresh();
	}

	/**
	 * Toggles display of toolbar on and off.
	 */
	public static void ToggleToolbar() {
		
		Text displayText = (Text)Misc.getItemContainingData("OptionViewToolbar",FrameIO.LoadFrame(Interfaces.TOGGLE_TOOLS_OVERLAY));
				
		if(DISPLAY_TOOLBAR){
			displayText.setText("View Toolbar");
			swapLinks(Interfaces.BLANK_TOOLBAR_OVERLAY,"overlayToolbar");
			DISPLAY_TOOLBAR = false;
			
		}else{
			
			displayText.setText("Hide Toolbar");
			swapLinks(Interfaces.NAVIGATION_OVERLAY_VIEW_NOTES_UNCLICKABLE,"overlayToolbar");
			DISPLAY_TOOLBAR = true;
		}
		
		FrameKeyboardActions.Refresh();
	}
	

	public static void ToggleNotesDisplay(CollectionItem collectItem) {
		

		//get link to the collection item's note frame.
		NoteLayer nl = collectItem.getNoteLayer();
		
		if(nl == null){
			MessageBay.errorMessage("No notes to display");
			return;
		}
		
		
		String noteFrameLink = nl.getFrame().getName();
		System.err.println("*** link (in ToggleNotesDisplay method): " + noteFrameLink);
		
		//Text displayText = (Text)Misc.getItemContainingData("OptionViewNotes",FrameIO.LoadFrame(Interfaces.NAVIGATION_OVERLAY_VIEW_NOTES_UNCLICKABLE));
		Text displayText = (Text)Misc.getItemContainingData("OptionViewNotes",FrameIO.LoadFrame(Interfaces.NAVIGATION_OVERLAY_VIEW_NOTES_CLICKABLE));
			
		if(DISPLAY_NOTES){
			System.err.println("Notes are NOT visible");
			displayText.setText("View Notes");
			//swapLinks(noteFrameLink,Interfaces.BLANK_HIDE_NOTES_OVERLAY);
			swapLinks(Interfaces.BLANK_HIDE_NOTES_OVERLAY,"overlayNotes");
			DISPLAY_NOTES = false;
				
					
		}else{
			displayText = (Text)Misc.getItemContainingData("OptionViewNotes",FrameIO.LoadFrame(Interfaces.NAVIGATION_OVERLAY_VIEW_NOTES_CLICKABLE));
			displayText.setText("Hide Notes");

			swapLinks(noteFrameLink,"overlayNotes");
			DISPLAY_NOTES = true;
		}
		
		FrameKeyboardActions.Refresh();
		
	}
	
	public static void fixTextItemWidths(Frame curr){
		
		List<Text> textItems = curr.getBodyTextItems(true);
		
		int i = textItems.size() -1;
		
		while(i > -1){
			Text t = textItems.get(i);
			if(t.getAbsoluteWidth() != null){
				if(t.getAbsoluteWidth() != Integer.MAX_VALUE && t.getAbsoluteWidth() > 0){
					Text copy = t.copy();
					curr.addItem(copy);
					curr.removeItem(t);
				}
			}
			i--;
		}
	}
	
	/**
	 * Used by toggleToolbar() and ToggleLayersDisplay() and ToggleNotesDisplay()
	 * to change the overlay annotation links. So if we want
	 * the display of these things off, the overlay will just be 
	 * a blank frame. Otherwise the overlay will be the appropriate
	 * frame displaying the toolbar/layer display.
	 * @param oldLink
	 * @param newLink
	 */
	private static void swapLinks(String newLink,String data){
		
		//TODO: Change this code so that it finds an item containing a data value, i.e. "overlayNotes", "overlayToolbar" or "overlayViewToolbar"
		//The item whose link we want to change
		Text overlayLink = (Text)Misc.getItemContainingData(data, DisplayIO.getCurrentFrame());
		
		overlayLink.setLink(newLink);
	}
}
